﻿using FlirLepton.Image;
using System.IO.Ports;
using System.Threading;

namespace FlirLepton.Communication
{
    public class LeptonConnection
    {
        private int leptonRows;
        private int leptonCols;

        private SerialPort serialPort;
        private Thread connectionThread;
        private static int currentRow;
        private static bool isSyncNeeded;

        public bool IsPendingResultRequest { get; private set; }
        public bool IsResultReady { get; private set; }
        private LeptonImage receivingImage;
        private LeptonImage resultImage;

        public LeptonConnection(string portName, int leptonRows, int leptonCols)
        {
            this.leptonCols = leptonCols;
            this.leptonRows = leptonRows;
            serialPort = new SerialPort(portName, 9600, Parity.None, 8, StopBits.One);
            connectionThread = new Thread(ReceiveData);
            currentRow = 0;
            isSyncNeeded = true;
            receivingImage = new LeptonImage(leptonRows, leptonCols);
        }

        public void Start()
        {
            serialPort.Open();
            connectionThread.Start();
        }

        public void Stop()
        {
            connectionThread.Abort();
            serialPort.Close();
        }

        private void ReceiveData()
        {
            while (true)
            {
                if (isSyncNeeded)
                {
                    Sync();
                }

                if (serialPort.BytesToRead >= 160)
                {
                    byte[] buffer = new byte[160];
                    serialPort.Read(buffer, 0, 160);
                    ProcessRow(buffer);
                }
                else
                {
                    Thread.Sleep(5);
                }
            }
            
        }

        private void Sync()
        {
            bool synchronized = false;
            while (true)
            {
                if (serialPort.BytesToRead > 0)
                {
                    byte syncByte = (byte)serialPort.ReadByte();
                    if (syncByte == 0 && synchronized)
                    {
                        isSyncNeeded = false;
                        break;
                    }

                    synchronized = syncByte == 0;
                }
                else
                {
                    Thread.Sleep(5);
                }
            }
        }

        private void ProcessRow(byte[] data)
        {
            for (int i = 0; i < leptonCols; i++)
            {
                ushort tempReading = (ushort)(data[2 * i] << 8 | data[2 * i + 1]);
                receivingImage.SetTempReading(currentRow, i, tempReading);
            }
            currentRow++;

            if (currentRow == leptonRows)
            {
                FrameHasBeenReceived();
            }
        }

        private void FrameHasBeenReceived()
        {
            isSyncNeeded = true;
            currentRow = 0;
            if (IsPendingResultRequest)
            {
                resultImage = receivingImage;
                IsPendingResultRequest = false;
                IsResultReady = true;
            }

            receivingImage = new LeptonImage(leptonRows, leptonCols);
        }

        public void RequestResult()
        {
            IsPendingResultRequest = true;
        }

        public LeptonImage GetResult()
        {
            if (!IsResultReady)
            {
                throw new LeptonResultException();
            }
            IsResultReady = false;
            return resultImage;
        }
    }
}