﻿using FlirLepton.Communication;
using FlirLepton.Image.LinearColors;
using System;
using System.Diagnostics;
using System.Drawing;
using System.Windows.Forms;

namespace FlirLepton.Image
{
    public class LeptonDraw : Control
    {
        private int leptonRows;
        private int leptonCols;

        private LeptonImage image;
        private TempReading maxTemp;
        private TempReading minTemp;
        private LinearColor linearColor;
        private Log log;
        private int fpsCounter;
        private double fps;

        private Stopwatch fpsSw = new Stopwatch();
        private Bitmap imageBitmap;

        public ColorPalettes ColorPallete { get; set; }

        private double PixelSize 
        {
            get
            {
                double widthSize = Width / (double)leptonCols;
                double heightSize = Height / (double)leptonRows;

                return widthSize > heightSize ? heightSize : widthSize;
            } 
        }

        public LeptonDraw(int leptonRows, int leptonCols, Log log)
        {
            this.log = log;
            this.leptonRows = leptonRows;
            this.leptonCols = leptonCols;
            DoubleBuffered = true;
            Dock = DockStyle.Fill;

            fpsCounter = 0;
            fpsSw.Start();
        }

        protected override void OnMouseClick(MouseEventArgs e)
        {
            int row = (int)(e.Y / PixelSize);
            int col = (int)(e.X / PixelSize);

            double temp = image.GetTempReading(row, col);
            log.LogMessage($"t({row}, {col}) = {temp} °C");
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            if (image != null)
            {
                e.Graphics.DrawImage(imageBitmap, 0, 0, (float)PixelSize * leptonCols, (float)PixelSize * leptonRows);
                DrawInfo(e.Graphics);
            }
        }

        private void DrawPixel(Graphics graphics, int row, int col, double value)
        {
            Color color = linearColor.GetColor(value);
            SolidBrush brush = new SolidBrush(color);
            graphics.FillRectangle(brush, col, row, 1, 1);
        }

        private void DrawInfo(Graphics graphics)
        {
            Font font = new Font("Consolas", 8);
            if (fpsSw.ElapsedMilliseconds >= 2000)
            {
                fps = fpsCounter / (fpsSw.ElapsedMilliseconds / (double)1000);
                fps = Math.Round(fps, 2);
                fpsCounter = 0;
                fpsSw.Restart();
            }

            string info = String.Format("FPS:      {0}\r\nMin temp: {1} °C\r\nMax temp: {2} °C", fps, minTemp.Temp, maxTemp.Temp);
            graphics.DrawString(info, font, Brushes.White, 0, 0);
        }

        protected override void OnSizeChanged(EventArgs e)
        {
            Invalidate();
        }

        private void DrawBitmap()
        {
            imageBitmap = new Bitmap(leptonCols, leptonRows);
            Graphics graphics = Graphics.FromImage(imageBitmap);
            for (int col = 0; col < leptonCols; col++)
            {
                for (int row = 0; row < leptonRows; row++)
                {
                    double temp = image.GetTempReading(row, col);
                    DrawPixel(graphics, row, col, temp);
                }
            }
        }

        public void SetImage(LeptonImage image)
        {
            fpsCounter++;
            this.image = image;
            maxTemp = image.GetMaxTemp();
            minTemp = image.GetMinTemp();
            linearColor = new LinearColorFactory().GetLinearColor(ColorPallete, minTemp.Temp, maxTemp.Temp);

            DrawBitmap();
            Invalidate();
        }

        public Bitmap GetBitmap()
        {
            return imageBitmap;
        }
    }
}